@@ -40,6 +40,7 @@
 
 // SCSI Architecture Model Family includes
 #include "SCSITask.h"
+#include "SCSILibraryRoutines.h"
 #include "SCSICmds_INQUIRY_Definitions.h"
 #include "IOSCSIPeripheralDeviceNub.h"
 #include "SCSITaskLib.h"
@@ -129,14 +130,10 @@ bool
 IOSCSIPeripheralDeviceNub::start ( IOService * provider )
 {
 	
-	OSDictionary * 	characterDict 	= NULL;
+	OSDictionary *	characterDict	= NULL;
 	OSObject *		obj				= NULL;
 	bool			result			= false;
 	
-	// Call our super class' start routine so that all inherited
-	// behavior is initialized.	
-	require ( super::start ( provider ), ErrorExit );
-	
 	// Check if our provider is one of us. If it is, we don't want to
 	// load again.
 	fProvider = OSDynamicCast ( IOSCSIPeripheralDeviceNub, provider );
@@ -146,12 +143,17 @@ IOSCSIPeripheralDeviceNub::start ( IOService * provider )
 	fProvider = OSDynamicCast ( IOSCSIProtocolInterface, provider );
 	require_nonzero_quiet ( fProvider, ErrorExit );
 	
+	// Call our super class' start routine so that all inherited
+	// behavior is initialized.
+	require ( super::start ( provider ), ErrorExit );
+	
 	fSCSIPrimaryCommandObject = OSTypeAlloc ( SCSIPrimaryCommands );
 	require_nonzero ( fSCSIPrimaryCommandObject, ErrorExit );
 	
 	require ( fProvider->open ( this ), ReleaseCommandObject );
 	
 	STATUS_LOG ( ( "%s: Check for the SCSI Device Characteristics property from provider.\n", getName ( ) ) );
+	
 	// Check if the personality for this device specifies a preferred protocol
 	if ( ( fProvider->getProperty ( kIOPropertySCSIDeviceCharacteristicsKey ) ) == NULL )
 	{
@@ -183,21 +185,21 @@ IOSCSIPeripheralDeviceNub::start ( IOService * provider )
 		else
 		{
 			
-			OSNumber *	defaultInquiry;
+			OSNumber *	defaultInquiry = NULL;
 			
 			STATUS_LOG ( ( "%s: Get Inquiry Length property.\n", getName ( ) ) );
 			defaultInquiry = OSDynamicCast ( OSNumber, characterDict->getObject ( kIOPropertySCSIInquiryLengthKey ) );
 			
 			// This device has a preferred protocol, use that.
 			fDefaultInquiryCount = defaultInquiry->unsigned32BitValue ( );
-		
+			
 		}
-	
-	}
 		
+	}
+	
 	STATUS_LOG ( ( "%s: default inquiry count is: %d\n", getName ( ), fDefaultInquiryCount ) );
 	
-	require_quiet ( InterrogateDevice ( ), CloseProvider );
+	require ( InterrogateDevice ( ), CloseProvider );
 	
 	setProperty ( kIOMatchCategoryKey, kSCSITaskUserClientIniterKey );
 	
@@ -209,7 +211,7 @@ IOSCSIPeripheralDeviceNub::start ( IOService * provider )
 	
 	else
 	{
-		characterDict->retain ( );
+		characterDict = OSDictionary::withDictionary ( characterDict );
 	}
 	
 	obj = fProvider->getProperty ( kIOPropertyPhysicalInterconnectTypeKey );
@@ -224,7 +226,7 @@ IOSCSIPeripheralDeviceNub::start ( IOService * provider )
 		characterDict->setObject ( kIOPropertyPhysicalInterconnectLocationKey, obj );
 	}
 	
-	obj = fProvider->getProperty ( kIOPropertyReadTimeOutDurationKey );	
+	obj = fProvider->getProperty ( kIOPropertyReadTimeOutDurationKey ); 
 	if ( obj != NULL );
 	{
 		characterDict->setObject ( kIOPropertyReadTimeOutDurationKey, obj );
@@ -307,7 +309,7 @@ ErrorExit:
 //
 
 IOReturn
-IOSCSIPeripheralDeviceNub::message ( UInt32 		type,
+IOSCSIPeripheralDeviceNub::message ( UInt32			type,
 									 IOService *	nub,
 									 void *			arg )
 {
@@ -324,10 +326,12 @@ IOSCSIPeripheralDeviceNub::message ( UInt32 		type,
 		{
 			
 			STATUS_LOG ( ("%s: kIOMessageServiceIsRequestingClose called\n", getName( ) ) );
-			if ( fProvider )
+			if ( fProvider != NULL )
 			{	
+				
 				STATUS_LOG ( ("%s: closing provider\n", getName( ) ) );
 				fProvider->close ( this );
+				
 			}
 			
 		}
@@ -363,25 +367,6 @@ IOSCSIPeripheralDeviceNub::message ( UInt32 		type,
 #endif
 
 
-//
-//	 joinPMtree - Relays joinPMtree call to provider.				[PROTECTED]
-//
-
-void
-IOSCSIPeripheralDeviceNub::joinPMtree ( IOService * driver )
-{
-	
-	STATUS_LOG ( ( "%s%s::%s called%s\n", "\033[33m",
-					getName ( ), __FUNCTION__, "\033[0m" ) );
-	
-	fProvider->joinPMtree ( driver );
-	
-	STATUS_LOG ( ( "%s%s::%s finished%s\n", "\033[33m",
-					getName ( ), __FUNCTION__, "\033[0m" ) );
-	
-}
-
-
 //
 //	 InitializePowerManagement - Does nothing.						[PROTECTED]
 //
@@ -412,12 +397,12 @@ IOSCSIPeripheralDeviceNub::InitializePowerManagement ( IOService * provider )
 //
 
 bool
-IOSCSIPeripheralDeviceNub::matchPropertyTable (	OSDictionary *	table,
+IOSCSIPeripheralDeviceNub::matchPropertyTable ( OSDictionary *	table,
 												SInt32 *		score )
 {
 	
-	bool 	returnValue 	= true;
-	bool 	isMatch 		= false;
+	bool	returnValue		= true;
+	bool	isMatch			= false;
 	SInt32	propertyScore	= *score;
 	
 	// Clamp the copy of the score to 4000 to prevent us from promoting
@@ -544,24 +529,24 @@ IOSCSIPeripheralDeviceNub::matchPropertyTable (	OSDictionary *	table,
 
 
 //
-//	 sCompareIOProperty - Compares properties.				[STATIC][PROTECTED]
+//	 sCompareIOProperty - Compares properties.				  [STATIC][PRIVATE]
 //
 
 bool
 IOSCSIPeripheralDeviceNub::sCompareIOProperty (
-								IOService * 		object,
-								OSDictionary * 		table,
-								char * 				propertyKeyName,
-								bool * 				matches )
+								IOService *			object,
+								OSDictionary *		table,
+								char *				propertyKeyName,
+								bool *				matches )
 {
 	
 	OSObject *	tableObject		= NULL;
 	OSObject *	deviceObject	= NULL;
-	bool 		returnValue		= false;
+	bool		returnValue		= false;
 	
 	*matches = false;
 	
-	tableObject  = table->getObject ( propertyKeyName );
+	tableObject	 = table->getObject ( propertyKeyName );
 	deviceObject = object->getProperty ( propertyKeyName );
 	
 	require_nonzero_quiet ( deviceObject, ErrorExit );
@@ -594,7 +579,7 @@ ErrorExit:
 bool
 IOSCSIPeripheralDeviceNub::SendSCSICommand (
 							SCSITaskIdentifier		request,
-							SCSIServiceResponse * 	serviceResponse,
+							SCSIServiceResponse *	serviceResponse,
 							SCSITaskStatus *		taskStatus )
 {
 	return false;
@@ -624,45 +609,71 @@ IOSCSIPeripheralDeviceNub::ExecuteCommand ( SCSITaskIdentifier request )
 }
 
 
-// The Task Management function to allow the SCSI Application Layer client to request
-// that a specific task be aborted.
-SCSIServiceResponse		
-IOSCSIPeripheralDeviceNub::AbortTask( UInt8 theLogicalUnit, SCSITaggedTaskIdentifier theTag )
+//
+//	 AbortTask - Relays command to protocol services driver.		[PROTECTED]
+//
+
+SCSIServiceResponse
+IOSCSIPeripheralDeviceNub::AbortTask ( UInt8 theLogicalUnit, SCSITaggedTaskIdentifier theTag )
 {
-	return 	fProvider->AbortTask ( theLogicalUnit, theTag );
+	return	fProvider->AbortTask ( theLogicalUnit, theTag );
 }
 
-// The Task Management function to allow the SCSI Application Layer client to request
-// that a all tasks curerntly in the task set be aborted.
-SCSIServiceResponse		
-IOSCSIPeripheralDeviceNub::AbortTaskSet( UInt8 theLogicalUnit )
-{
-	return 	fProvider->AbortTaskSet ( theLogicalUnit );
-}
+
+//
+//	 AbortTaskSet - Relays command to protocol services driver.	[PROTECTED]
+//
 
 SCSIServiceResponse		
-IOSCSIPeripheralDeviceNub::ClearACA( UInt8 theLogicalUnit )
+IOSCSIPeripheralDeviceNub::AbortTaskSet ( UInt8 theLogicalUnit )
 {
-	return 	fProvider->ClearACA ( theLogicalUnit );
+	return	fProvider->AbortTaskSet ( theLogicalUnit );
 }
 
-SCSIServiceResponse		
-IOSCSIPeripheralDeviceNub::ClearTaskSet( UInt8 theLogicalUnit )
+
+//
+//	 ClearACA - Relays command to protocol services driver.		[PROTECTED]
+//
+
+SCSIServiceResponse
+IOSCSIPeripheralDeviceNub::ClearACA ( UInt8 theLogicalUnit )
 {
-	return 	fProvider->ClearTaskSet ( theLogicalUnit );
-}
-    
-SCSIServiceResponse		
-IOSCSIPeripheralDeviceNub::LogicalUnitReset( UInt8 theLogicalUnit )
-{
-	return 	fProvider->LogicalUnitReset ( theLogicalUnit );
+	return	fProvider->ClearACA ( theLogicalUnit );
 }
 
-SCSIServiceResponse		
-IOSCSIPeripheralDeviceNub::TargetReset( void )
+
+//
+//	 ClearTaskSet - Relays command to protocol services driver.	[PROTECTED]
+//
+
+SCSIServiceResponse
+IOSCSIPeripheralDeviceNub::ClearTaskSet ( UInt8 theLogicalUnit )
 {
-	return 	fProvider->TargetReset ( );
+	return	fProvider->ClearTaskSet ( theLogicalUnit );
 }
+	
+
+//
+//	 LogicalUnitReset - Relays command to protocol services driver.[PROTECTED]
+//
+
+SCSIServiceResponse
+IOSCSIPeripheralDeviceNub::LogicalUnitReset ( UInt8 theLogicalUnit )
+{
+	return	fProvider->LogicalUnitReset ( theLogicalUnit );
+}
+
+
+//
+//	 TargetReset - Relays command to protocol services driver.		[PROTECTED]
+//
+
+SCSIServiceResponse
+IOSCSIPeripheralDeviceNub::TargetReset ( void )
+{
+	return	fProvider->TargetReset ( );
+}
+
 
 //
 //	 AbortCommand - Relays command to protocol services driver.	[PROTECTED]
@@ -705,33 +716,30 @@ IOSCSIPeripheralDeviceNub::HandleProtocolServiceFeature (
 
 #if 0
 #pragma mark -
-#pragma mark  Private method declarations
+#pragma mark  Private Methods
 #pragma mark -
 #endif
 
 
 //
-//	 TaskCallback - Callback method for synchronous commands.		  [PRIVATE]
+//	 TaskCallback - Callback method for synchronous commands.
+//															  [STATIC][PRIVATE]
 //
 
 void 
 IOSCSIPeripheralDeviceNub::TaskCallback ( SCSITaskIdentifier completedTask )
 {
 	
-	IOSyncer *				fSyncLock		= NULL;
-	SCSITask *				scsiRequest		= NULL;
-	SCSIServiceResponse 	serviceResponse;
+	SCSITask *						scsiRequest = NULL;
+	IOSCSIPeripheralDeviceNub *		owner		= NULL;
 	
-	STATUS_LOG ( ( "IOSCSIPeripheralDeviceNub::TaskCallback called\n.") );
-		
 	scsiRequest = OSDynamicCast ( SCSITask, completedTask );
-	require_nonzero_string ( scsiRequest, ErrorExit, "SCSITask is NULL\n" );
+	require_nonzero ( scsiRequest, ErrorExit );
 	
-	fSyncLock = ( IOSyncer * ) scsiRequest->GetApplicationLayerReference ( );
-	require_nonzero_string ( fSyncLock, ErrorExit, "fSyncLock is NULL\n" );
+	owner = ( IOSCSIPeripheralDeviceNub * ) scsiRequest->GetTaskOwner ( );
+	require_nonzero ( owner, ErrorExit );
 	
-	serviceResponse = scsiRequest->GetServiceResponse ( );
-	fSyncLock->signal ( serviceResponse, false );
+	owner->TaskCompletion ( completedTask );
 	
 	
 ErrorExit:
@@ -743,40 +751,79 @@ ErrorExit:
 
 
 //
-//	 SendTask - Method to send synchronous commands.				  [PRIVATE]
+//	 TaskCompletion - Callback method for synchronous commands.	  [PRIVATE]
 //
 
-SCSIServiceResponse
-IOSCSIPeripheralDeviceNub::SendTask ( SCSITask * request )
+void 
+IOSCSIPeripheralDeviceNub::TaskCompletion ( SCSITaskIdentifier completedTask )
 {
 	
-	SCSIServiceResponse 	serviceResponse = kSCSIServiceResponse_SERVICE_DELIVERY_OR_TARGET_FAILURE;
-	IOSyncer *				fSyncLock		= NULL;
-	
-	fSyncLock = IOSyncer::create ( false );
-	require_nonzero_string ( fSyncLock, ErrorExit, "IOSyncer is NULL\n" );
-	
-	fSyncLock->signal ( kIOReturnSuccess, false );
-	
-	request->SetTaskCompletionCallback ( &IOSCSIPeripheralDeviceNub::TaskCallback );
-	request->SetApplicationLayerReference ( ( void * ) fSyncLock );
-	fSyncLock->reinit ( );
-	
-	ExecuteCommand ( request );
-	
-	// Wait for the completion routine to get called
-	serviceResponse = ( SCSIServiceResponse ) fSyncLock->wait ( false );
-	fSyncLock->release ( );
-	
-	
-ErrorExit:
-	
-	
-	return serviceResponse;
+	fCommandGate->commandWakeup ( completedTask, true );
 	
 }
 
 
+//
+//  sWaitForTask - Called to wait for a task until it completes.
+//															  [STATIC][PRIVATE]
+//
+
+IOReturn
+IOSCSIPeripheralDeviceNub::sWaitForTask (
+									void *		object,
+									SCSITask *	request )
+{
+	
+	IOSCSIPeripheralDeviceNub * device = NULL;
+	
+	device = OSDynamicCast ( IOSCSIPeripheralDeviceNub, ( OSObject * ) object );
+	
+	return device->GatedWaitForTask ( request );
+	
+}
+
+
+
+
+//
+//  GatedWaitForTask - Called to wait for a task to complete.		  [PRIVATE]
+//
+
+IOReturn
+IOSCSIPeripheralDeviceNub::GatedWaitForTask ( SCSITask * request )
+{
+	
+	IOReturn	result	= kIOReturnBadArgument;
+	
+	if ( request != NULL )
+	{
+		
+		while ( request->GetTaskState ( ) != kSCSITaskState_ENDED )
+		{
+			
+			result = fCommandGate->commandSleep ( request, THREAD_UNINT );
+			
+		}
+		
+	}
+	
+	if ( result == THREAD_AWAKENED )
+		result = kIOReturnSuccess;
+	
+	return result;
+	
+}
+
+
+
+
+#if 0
+#pragma mark -
+#pragma mark  Protected Methods
+#pragma mark -
+#endif
+
+
 //
 //	 InterrogateDevice - Method to interrogate device.				  [PRIVATE]
 //
@@ -786,18 +833,18 @@ IOSCSIPeripheralDeviceNub::InterrogateDevice ( void )
 {
 	
 	OSString *						string			= NULL;
-	SCSICmd_INQUIRY_StandardData * 	inqData 		= NULL;
-	SCSIServiceResponse 			serviceResponse	= kSCSIServiceResponse_SERVICE_DELIVERY_OR_TARGET_FAILURE;
+	SCSICmd_INQUIRY_StandardData *	inqData			= NULL;
+	SCSIServiceResponse				serviceResponse = kSCSIServiceResponse_SERVICE_DELIVERY_OR_TARGET_FAILURE;
 	SCSI_Sense_Data					senseBuffer		= { 0 };
-	IOMemoryDescriptor *			bufferDesc 		= NULL;
-	SCSITask *						request 		= NULL;
+	IOMemoryDescriptor *			bufferDesc		= NULL;
+	SCSITask *						request			= NULL;
 	UInt8							inqDataCount	= 0;
 	int								index			= 0;
 	bool							result			= false;
-	char							tempString[17]; // Maximum + 1 for null char
+	char							tempString[17]	= { 0 }; // Maximum + 1 for null char
 	
-   	// Before we register ourself as a nub, we need to find out what 
-   	// type of device we want to connect to us
+	// Before we register ourself as a nub, we need to find out what 
+	// type of device we want to connect to us
 	// Do an Inquiry command and parse the data to determine the peripheral
 	// device type.
 	if ( fDefaultInquiryCount == 0 )
@@ -816,6 +863,7 @@ IOSCSIPeripheralDeviceNub::InterrogateDevice ( void )
 		// This device has a default inquiry count, use it.
 		STATUS_LOG ( ( "%s: use fDefaultInquiryCount for Inquiry.\n", getName ( ) ) );
 		inqDataCount = fDefaultInquiryCount;
+		check ( inqDataCount >= sizeof ( SCSICmd_INQUIRY_StandardData ) );
 		
 	}
 	
@@ -831,16 +879,17 @@ IOSCSIPeripheralDeviceNub::InterrogateDevice ( void )
 	request = OSTypeAlloc ( SCSITask );	
 	require_nonzero ( request, ReleaseBuffer );
 	request->ResetForNewTask ( );
-	
 	fSCSIPrimaryCommandObject->TEST_UNIT_READY ( request, 0x00 );
+	request->SetTimeoutDuration ( 10000 );
 	
 	// The command was successfully built, now send it
 	serviceResponse = SendTask ( request );
+	
 	if ( serviceResponse == kSCSIServiceResponse_TASK_COMPLETE )
 	{
 		
 		bool validSense = false;
-
+		
 		if ( request->GetTaskStatus ( ) == kSCSITaskStatus_CHECK_CONDITION )
 		{
 			
@@ -853,6 +902,7 @@ IOSCSIPeripheralDeviceNub::InterrogateDevice ( void )
 														   bufferDesc,
 														   kSenseDefaultSize,
 														   0 );
+				request->SetTimeoutDuration ( 10000 );
 				serviceResponse = SendTask ( request );
 				
 				if ( ( serviceResponse == kSCSIServiceResponse_TASK_COMPLETE ) &&
@@ -884,12 +934,14 @@ IOSCSIPeripheralDeviceNub::InterrogateDevice ( void )
 				{
 					
 					// Check the sense data to see if the TUR was sent to an invalid LUN and if so,
-					// abort trying to access this Logical Unit.
-					if ( ( ( senseBuffer.SENSE_KEY & kSENSE_KEY_Mask ) == kSENSE_KEY_ILLEGAL_REQUEST ) &&
-						   ( senseBuffer.ADDITIONAL_SENSE_CODE == 0x25 ) &&
-						   ( senseBuffer.ADDITIONAL_SENSE_CODE_QUALIFIER == 0x00 ) )
-				 	{
-				 		
+					// abort trying to access this Logical Unit. We used to check the sense key for
+					// ILLEGAL_REQUEST, but some devices which aren't spun up yet will set NOT_READY
+					// for the SENSE_KEY. Might as well not use it...
+					if ( ( senseBuffer.ADDITIONAL_SENSE_CODE == 0x25 ) &&
+						 ( senseBuffer.ADDITIONAL_SENSE_CODE_QUALIFIER == 0x00 ) )
+					{
+						
+						ERROR_LOG ( ( "ASC/ASCQ = 0x25/0x00 - Invalid LUN\n" ) );
 						goto ReleaseTask;
 						
 					}
@@ -902,12 +954,30 @@ IOSCSIPeripheralDeviceNub::InterrogateDevice ( void )
 		
 	}
 	
- 	IOSleep ( 100 );
- 	
- 	// Check if we got terminated. If so, bail early.
- 	require ( ( isInactive ( ) == false ), ReleaseTask );
- 	
-   	for ( index = 0; ( index < kMaxInquiryAttempts ) && ( isInactive ( ) == false ); index++ )
+	else if ( ( serviceResponse == kSCSIServiceResponse_SERVICE_DELIVERY_OR_TARGET_FAILURE ) &&
+			  ( request->GetTaskStatus ( ) == kSCSITaskStatus_DeviceNotResponding ) )
+	{
+		
+		ERROR_LOG ( ( "taskStatus = DeviceNotResponding\n" ) );
+		goto ReleaseTask;
+		
+	}
+
+	else if ( ( serviceResponse == kSCSIServiceResponse_SERVICE_DELIVERY_OR_TARGET_FAILURE ) &&
+			  ( request->GetTaskStatus ( ) == kSCSITaskStatus_DeviceNotPresent ) )
+	{
+		
+		ERROR_LOG ( ( "taskStatus = DeviceNotPresent\n" ) );
+		goto ReleaseTask;
+		
+	}
+	
+	IOSleep ( 100 );
+	
+	// Check if we got terminated. If so, bail early.
+	require ( ( isInactive ( ) == false ), ReleaseTask );
+	
+	for ( index = 0; ( index < kMaxInquiryAttempts ) && ( isInactive ( ) == false ); index++ )
 	{
 		
 		request->ResetForNewTask ( );
@@ -915,11 +985,12 @@ IOSCSIPeripheralDeviceNub::InterrogateDevice ( void )
 									request,
 									bufferDesc,
 									0,
-								  	0,
+									0,
 									0,
 									inqDataCount,
 									0 );
-			
+		request->SetTimeoutDuration ( 10000 );
+		
 		serviceResponse = SendTask ( request );
 		
 		if ( ( serviceResponse == kSCSIServiceResponse_TASK_COMPLETE ) &&
@@ -928,86 +999,105 @@ IOSCSIPeripheralDeviceNub::InterrogateDevice ( void )
 			break;	
 		}
 		
-		else
+		else if ( ( serviceResponse == kSCSIServiceResponse_SERVICE_DELIVERY_OR_TARGET_FAILURE ) &&
+				  ( request->GetTaskStatus ( ) == kSCSITaskStatus_DeviceNotResponding ) )
 		{
-			IOSleep ( 1000 );
+			
+			ERROR_LOG ( ( "taskStatus = DeviceNotResponding\n" ) );
+			goto ReleaseTask;
+			
 		}
 		
+		else if ( ( serviceResponse == kSCSIServiceResponse_SERVICE_DELIVERY_OR_TARGET_FAILURE ) &&
+				  ( request->GetTaskStatus ( ) == kSCSITaskStatus_DeviceNotPresent ) )
+		{
+			
+			ERROR_LOG ( ( "taskStatus = DeviceNotPresent\n" ) );
+			goto ReleaseTask;
+			
+		}
+		
+		else if ( ( serviceResponse == kSCSIServiceResponse_TASK_COMPLETE ) &&
+				  ( request->GetTaskStatus ( ) == kSCSITaskStatus_CHECK_CONDITION ) )
+		{
+			
+			ERROR_LOG ( ( "CHECK_CONDITION on INQUIRY command. THIS SHOULD NEVER HAPPEN! BAD DEVICE, NO COOKIE!!\n" ) );
+			
+		}
+		
+		IOSleep ( 1000 );
+		
 	}
 	
 	// Check if we have exhausted our max number of attempts. If so, bail early.
 	require ( ( index != kMaxInquiryAttempts ), ReleaseTask );
- 	
- 	// Check if we got terminated. If so, bail early.
- 	require ( isInactive ( ) == false, ReleaseTask );
+	
+	// Check if we got terminated. If so, bail early.
+	require ( isInactive ( ) == false, ReleaseTask );
+	
+	require ( ( request->GetRealizedDataTransferCount ( ) >= ( sizeof ( SCSICmd_INQUIRY_StandardData ) ) ), ReleaseTask );
 	
 	{
-        
-		UInt8	qualifier = inqData->PERIPHERAL_DEVICE_TYPE & kINQUIRY_PERIPHERAL_QUALIFIER_Mask;
-            
-		if ( qualifier & kINQUIRY_PERIPHERAL_QUALIFIER_SupportedButNotConnected )
-			goto ReleaseTask;
 		
-		if ( qualifier & kINQUIRY_PERIPHERAL_QUALIFIER_NotSupported )
-			goto ReleaseTask;
+		UInt8	qualifier = inqData->PERIPHERAL_DEVICE_TYPE & kINQUIRY_PERIPHERAL_QUALIFIER_Mask;
+		
+		switch ( qualifier )
+		{
+			
+			case kINQUIRY_PERIPHERAL_QUALIFIER_NotSupported:
+				// According to the SPC-2 spec, if a devices responds with this
+				// qualifier type, it should set the PDT to 0x1F for backward
+				// compatibility...
+				ERROR_LOG ( ( "Device not supported at this Logical Unit\n" ) );
+				check ( ( inqData->PERIPHERAL_DEVICE_TYPE & kINQUIRY_PERIPHERAL_TYPE_Mask ) == kINQUIRY_PERIPHERAL_TYPE_UnknownOrNoDeviceType );
+				goto ReleaseTask;
+				break;
+			
+			case kINQUIRY_PERIPHERAL_QUALIFIER_SupportedButNotConnected:
+				// Why do some devices respond with this qualifier type when they actually have
+				// a Logical Unit present at this Logical Unit Number? So many arrays
+				// do this, so we have to allow for it and not reject the LUN if it responds
+				// this way...
+				STATUS_LOG ( ( "Device supported but not connected\n" ) );
+				break;
+			
+			case kINQUIRY_PERIPHERAL_QUALIFIER_Connected:
+				// A device is definitely connected.
+				STATUS_LOG ( ( "Device supported and connected\n" ) );
+				break;
+				
+			default:
+				ERROR_LOG ( ( "Some reserved qualifier = 0x%02x\n", qualifier ) );
+				break;
+			
+		}
 		
 	}
-        
-   	// Set the Peripheral Device Type property for the device.
-   	setProperty ( kIOPropertySCSIPeripheralDeviceType,
-   				( UInt64 ) ( inqData->PERIPHERAL_DEVICE_TYPE & kINQUIRY_PERIPHERAL_TYPE_Mask ),
-   				kIOPropertySCSIPeripheralDeviceTypeSize );
 	
-   	// Set the Vendor Identification property for the device.
-   	for ( index = 0; index < kINQUIRY_VENDOR_IDENTIFICATION_Length; index++ )
-   	{
-   		tempString[index] = inqData->VENDOR_IDENTIFICATION[index];
-   	}
-	tempString[index] = 0;
+	// Set the Peripheral Device Type property for the device.
+	setProperty ( kIOPropertySCSIPeripheralDeviceType,
+				( UInt64 ) ( inqData->PERIPHERAL_DEVICE_TYPE & kINQUIRY_PERIPHERAL_TYPE_Mask ),
+				kIOPropertySCSIPeripheralDeviceTypeSize );
+	
+	// Set the Vendor Identification property for the device.
+	bcopy ( inqData->VENDOR_IDENTIFICATION, tempString, kINQUIRY_VENDOR_IDENTIFICATION_Length );
+	tempString[kINQUIRY_VENDOR_IDENTIFICATION_Length] = 0;
+	StripWhiteSpace ( tempString, kINQUIRY_VENDOR_IDENTIFICATION_Length );
 	
-   	for ( index = kINQUIRY_VENDOR_IDENTIFICATION_Length - 1; index != 0; index-- )
-   	{
-   		
-   		if ( tempString[index] != ' ' )
-   		{
-   			
-   			// Found a real character
-   			tempString[index + 1] = '\0';
-   			break;
-   			
-   		}
-   		
-   	}
-   	
 	string = OSString::withCString ( tempString );
 	if ( string != NULL )
 	{
 		
 		setProperty ( kIOPropertySCSIVendorIdentification, string );
 		string->release ( );
+		string = NULL;
 		
 	}
 	
-   	// Set the Product Indentification property for the device.
-   	for ( index = 0; index < kINQUIRY_PRODUCT_IDENTIFICATION_Length; index++ )
-   	{
-   		tempString[index] = inqData->PRODUCT_IDENTIFICATION[index];
-   	}
-   	tempString[index] = 0;
-	
-   	for ( index = kINQUIRY_PRODUCT_IDENTIFICATION_Length - 1; index != 0; index-- )
-   	{
-   		
-   		if ( tempString[index] != ' ' )
-   		{
-   			
-   			// Found a real character
-   			tempString[index+1] = '\0';
-   			break;
-   			
-   		}
-   		
-   	}
+	// Set the Product Indentification property for the device.
+	bcopy ( inqData->PRODUCT_IDENTIFICATION, tempString, kINQUIRY_PRODUCT_IDENTIFICATION_Length );
+	tempString[kINQUIRY_PRODUCT_IDENTIFICATION_Length] = 0;
+	StripWhiteSpace ( tempString, kINQUIRY_PRODUCT_IDENTIFICATION_Length );
 	
 	string = OSString::withCString ( tempString );
 	if ( string != NULL )
@@ -1015,29 +1105,14 @@ IOSCSIPeripheralDeviceNub::InterrogateDevice ( void )
 		
 		setProperty ( kIOPropertySCSIProductIdentification, string );
 		string->release ( );
+		string = NULL;
 		
 	}
 
-   	// Set the Product Revision Level property for the device.
-   	for ( index = 0; index < kINQUIRY_PRODUCT_REVISION_LEVEL_Length; index++ )
-   	{
-   		tempString[index] = inqData->PRODUCT_REVISION_LEVEL[index];
-   	}
-   	tempString[index] = 0;
-	
-   	for ( index = kINQUIRY_PRODUCT_REVISION_LEVEL_Length - 1; index != 0; index-- )
-   	{
-		
-		if ( tempString[index] != ' ' )
-		{
-			
-			// Found a real character
-			tempString[index+1] = '\0';
-			break;
-			
-		}
-		
-	}
+	// Set the Product Revision Level property for the device.
+	bcopy ( inqData->PRODUCT_REVISION_LEVEL, tempString, kINQUIRY_PRODUCT_REVISION_LEVEL_Length );
+	tempString[kINQUIRY_PRODUCT_REVISION_LEVEL_Length] = 0;
+	StripWhiteSpace ( tempString, kINQUIRY_PRODUCT_REVISION_LEVEL_Length );
 	
 	string = OSString::withCString ( tempString );
 	if ( string != NULL )
@@ -1045,6 +1120,7 @@ IOSCSIPeripheralDeviceNub::InterrogateDevice ( void )
 		
 		setProperty ( kIOPropertySCSIProductRevisionLevel, string );
 		string->release ( );
+		string = NULL;
 		
 	}
 	
@@ -1084,6 +1160,33 @@ ErrorExit:
 }
 
 
+//
+//	 SendTask - Method to send synchronous commands.				[PROTECTED]
+//
+
+SCSIServiceResponse
+IOSCSIPeripheralDeviceNub::SendTask ( SCSITask * request )
+{
+	
+	SCSIServiceResponse		serviceResponse = kSCSIServiceResponse_SERVICE_DELIVERY_OR_TARGET_FAILURE;
+	
+	request->SetTaskCompletionCallback ( &IOSCSIPeripheralDeviceNub::TaskCallback );
+	request->SetTaskOwner ( this );
+	
+	ExecuteCommand ( request );
+	
+	// Wait for the completion routine to get called
+	fCommandGate->runAction ( ( IOCommandGate::Action )
+							  &IOSCSIPeripheralDeviceNub::sWaitForTask,
+							  request );
+	
+	serviceResponse = request->GetServiceResponse ( );
+	
+	return serviceResponse;
+	
+}
+
+
 #if 0
 #pragma mark -
 #pragma mark  VTable Padding
@@ -1128,25 +1231,31 @@ bool
 IOSCSILogicalUnitNub::start ( IOService * provider )
 {
 	
-	OSDictionary * 	characterDict 	= NULL;
+	OSDictionary *	characterDict	= NULL;
 	OSNumber *		number			= NULL;
 	OSObject *		obj				= NULL;
 	bool			result			= false;
 	
-	require ( super::start ( provider ), ErrorExit );
-	
+	// Check if our provider is one of us. If it is, we don't want to
+	// load again.
 	fProvider = OSDynamicCast ( IOSCSIPeripheralDeviceNub, provider );
-	require_nonzero_quiet ( ( fProvider == NULL ), ErrorExit );
+	require_quiet ( ( fProvider == NULL ), ErrorExit );
 	
+	// Make sure our provider is a protocol services driver.
 	fProvider = OSDynamicCast ( IOSCSIProtocolInterface, provider );
 	require_nonzero_quiet ( fProvider, ErrorExit );
 	
+	// Call our super class' start routine so that all inherited
+	// behavior is initialized.
+	require ( super::start ( provider ), ErrorExit );
+	
 	fSCSIPrimaryCommandObject = OSTypeAlloc ( SCSIPrimaryCommands );
 	require_nonzero ( fSCSIPrimaryCommandObject, ErrorExit );
 	
 	require ( fProvider->open ( this ), ReleaseCommandObject );
 	
 	STATUS_LOG ( ( "%s: Check for the SCSI Device Characteristics property from provider.\n", getName ( ) ) );
+	
 	// Check if the personality for this device specifies a preferred protocol
 	if ( ( fProvider->getProperty ( kIOPropertySCSIDeviceCharacteristicsKey ) ) == NULL )
 	{
@@ -1178,21 +1287,21 @@ IOSCSILogicalUnitNub::start ( IOService * provider )
 		else
 		{
 			
-			OSNumber *	defaultInquiry;
+			OSNumber *	defaultInquiry = NULL;
 			
 			STATUS_LOG ( ( "%s: Get Inquiry Length property.\n", getName ( ) ) );
 			defaultInquiry = OSDynamicCast ( OSNumber, characterDict->getObject ( kIOPropertySCSIInquiryLengthKey ) );
 			
 			// This device has a preferred protocol, use that.
 			fDefaultInquiryCount = defaultInquiry->unsigned32BitValue ( );
-		
+			
 		}
-	
-	}
 		
+	}
+	
 	STATUS_LOG ( ( "%s: default inquiry count is: %d\n", getName ( ), fDefaultInquiryCount ) );
 	
-	require_quiet ( InterrogateDevice ( ), CloseProvider );
+	require ( InterrogateDevice ( ), CloseProvider );
 	
 	setProperty ( kIOMatchCategoryKey, kSCSITaskUserClientIniterKey );
 	
@@ -1308,6 +1417,18 @@ IOSCSILogicalUnitNub::SetLogicalUnitNumber ( UInt8 newLUN )
 }
 
 
+//
+//	 GetLogicalUnitNumber - Gets the logical unit number for this device.
+//																	[PROTECTED]
+//
+
+UInt8
+IOSCSILogicalUnitNub::GetLogicalUnitNumber ( void )
+{
+	return fLogicalUnitNumber;
+}
+
+
 //
 //	 ExecuteCommand - Relays command to protocol services driver.	[PROTECTED]
 //
@@ -1324,9 +1445,9 @@ IOSCSILogicalUnitNub::ExecuteCommand ( SCSITaskIdentifier request )
 		
 		SCSITask *	scsiRequest;
 		
-	    scsiRequest = OSDynamicCast ( SCSITask, request );
-	    if ( scsiRequest != NULL )
-	    {
+		scsiRequest = OSDynamicCast ( SCSITask, request );
+		if ( scsiRequest != NULL )
+		{
 			
 			scsiRequest->SetLogicalUnitNumber ( fLogicalUnitNumber );
 			
@@ -1358,15 +1479,15 @@ IOSCSILogicalUnitNub::AbortCommand ( SCSITaskIdentifier abortTask )
 
 
 // Space reserved for future expansion.
-OSMetaClassDefineReservedUnused ( IOSCSILogicalUnitNub,  1 );
-OSMetaClassDefineReservedUnused ( IOSCSILogicalUnitNub,  2 );
-OSMetaClassDefineReservedUnused ( IOSCSILogicalUnitNub,  3 );
-OSMetaClassDefineReservedUnused ( IOSCSILogicalUnitNub,  4 );
-OSMetaClassDefineReservedUnused ( IOSCSILogicalUnitNub,  5 );
-OSMetaClassDefineReservedUnused ( IOSCSILogicalUnitNub,  6 );
-OSMetaClassDefineReservedUnused ( IOSCSILogicalUnitNub,  7 );
-OSMetaClassDefineReservedUnused ( IOSCSILogicalUnitNub,  8 );
-OSMetaClassDefineReservedUnused ( IOSCSILogicalUnitNub,  9 );
+OSMetaClassDefineReservedUnused ( IOSCSILogicalUnitNub,	 1 );
+OSMetaClassDefineReservedUnused ( IOSCSILogicalUnitNub,	 2 );
+OSMetaClassDefineReservedUnused ( IOSCSILogicalUnitNub,	 3 );
+OSMetaClassDefineReservedUnused ( IOSCSILogicalUnitNub,	 4 );
+OSMetaClassDefineReservedUnused ( IOSCSILogicalUnitNub,	 5 );
+OSMetaClassDefineReservedUnused ( IOSCSILogicalUnitNub,	 6 );
+OSMetaClassDefineReservedUnused ( IOSCSILogicalUnitNub,	 7 );
+OSMetaClassDefineReservedUnused ( IOSCSILogicalUnitNub,	 8 );
+OSMetaClassDefineReservedUnused ( IOSCSILogicalUnitNub,	 9 );
 OSMetaClassDefineReservedUnused ( IOSCSILogicalUnitNub, 10 );
 OSMetaClassDefineReservedUnused ( IOSCSILogicalUnitNub, 11 );
 OSMetaClassDefineReservedUnused ( IOSCSILogicalUnitNub, 12 );
