@@ -78,7 +78,11 @@ private:
 										bool *			matches );
 	
 	static void		TaskCallback ( SCSITaskIdentifier completedTask );
-
+	void			TaskCompletion ( SCSITaskIdentifier completedTask );
+	
+	static IOReturn	sWaitForTask ( void * object, SCSITask * request );
+	IOReturn		GatedWaitForTask ( SCSITask * request );
+	
 protected:
 
 	SCSIServiceResponse SendTask ( SCSITask * request );
@@ -117,8 +121,6 @@ public:
 	
 	virtual IOReturn	message ( UInt32 type, IOService * nub, void * arg );
 										
-	virtual void		joinPMtree ( IOService * driver );
-	
 	virtual bool		matchPropertyTable ( OSDictionary * table,
 											 SInt32 * score );
 	
@@ -128,19 +130,19 @@ public:
 	
 	// The Task Management function to allow the SCSI Application Layer client to request
 	// that a specific task be aborted.
-	virtual SCSIServiceResponse		AbortTask( UInt8 theLogicalUnit, SCSITaggedTaskIdentifier theTag );
+	virtual SCSIServiceResponse		AbortTask ( UInt8 theLogicalUnit, SCSITaggedTaskIdentifier theTag );
 
 	// The Task Management function to allow the SCSI Application Layer client to request
 	// that a all tasks curerntly in the task set be aborted.
-	virtual SCSIServiceResponse		AbortTaskSet( UInt8 theLogicalUnit );
+	virtual SCSIServiceResponse		AbortTaskSet ( UInt8 theLogicalUnit );
 
-	virtual SCSIServiceResponse		ClearACA( UInt8 theLogicalUnit );
+	virtual SCSIServiceResponse		ClearACA ( UInt8 theLogicalUnit );
 
-	virtual SCSIServiceResponse		ClearTaskSet( UInt8 theLogicalUnit );
+	virtual SCSIServiceResponse		ClearTaskSet ( UInt8 theLogicalUnit );
     
-	virtual SCSIServiceResponse		LogicalUnitReset( UInt8 theLogicalUnit );
+	virtual SCSIServiceResponse		LogicalUnitReset ( UInt8 theLogicalUnit );
 
-	virtual SCSIServiceResponse		TargetReset( void );
+	virtual SCSIServiceResponse		TargetReset ( void );
 
     // ************* Obsoleted Member Routine ****************
     // The AbortCommand method is replaced by the AbortTask Management function and
@@ -192,6 +194,8 @@ public:
 	
 	virtual void		SetLogicalUnitNumber ( UInt8 newLUN );
 	
+	UInt8				GetLogicalUnitNumber ( void );
+	
 	// The ExecuteCommand method will take a SCSITask object and transport
 	// it across the physical wires to the device
 	virtual	void		ExecuteCommand ( SCSITaskIdentifier	request );
