@@ -52,7 +52,10 @@
 //
 
 #define DEBUG 0
+
 #define DEBUG_ASSERT_COMPONENT_NAME_STRING "CDTOC"
+
+#if DEBUG
 #define DEBUG_ASSERT_MESSAGE(componentNameString,	\
 							 assertionString,		\
 							 exceptionLabelString,	\
@@ -95,6 +98,8 @@ DebugAssert ( const char *	componentNameString,
 	
 }
 
+#endif	/* DEBUG */
+
 #include <AssertMacros.h>
 
 
