@@ -43,11 +43,14 @@
 #import "AuthoringDevice.h"
 
 #import <sys/mount.h>
+#import <libkern/OSByteOrder.h>
 #import <IOKit/IOBSD.h>
+#import <IOKit/IOKitLib.h>
 #import <IOKit/storage/IOMedia.h>
 #import <IOKit/scsi/SCSICommandOperationCodes.h>
 #import <IOKit/scsi/SCSITask.h>
-#import <Carbon/Carbon.h>
+#import <DiskArbitration/DiskArbitration.h>
+#import <CoreFoundation/CoreFoundation.h>
 
 
 //
@@ -61,12 +64,17 @@ enum
 	kStatusOK			= 0
 };
 
-#define	kNumBlocks				(1)
-#define	kRequestSize			(2048 * kNumBlocks)
-#define	kMyDocumentString		@"MyDocument"
-#define kAlertString			@"Alert"
-#define kOKString				@"OK"
-#define kCancelString			@"Cancel"
+
+#define	kNumBlocks					(1)
+#define	kRequestSize				(2048 * kNumBlocks)
+#define	kMyDocumentString			@"MyDocument"
+#define kAlertString				@"Alert"
+#define kOKString					@"OK"
+#define kCancelString				@"Cancel"
+#define IOBlockStorageDriverString	"IOBlockStorageDriver"
+
+static void
+DiskUnmountCallback ( DADiskRef disk, DADissenterRef dissenter, void * context );
 
 
 //
@@ -301,69 +309,26 @@ ErrorExit:
 {
 	
 	io_service_t	service 	= MACH_PORT_NULL;
-	io_service_t	child 		= MACH_PORT_NULL;
-	IOReturn		err			= kIOReturnSuccess;
-	int				result		= kStatusOK;
+	CFStringRef		bsdName		= NULL;
+	int				result		= kUnmountFailed;
 	
 	// Get a handle to the io_service_t that represents our MMC Device
 	service = [ [ self theAuthoringDeviceTester ] getServiceObject: [ self theAuthoringDevice ] ];
 	require ( ( service != MACH_PORT_NULL ), ErrorExit );
 	
-	// Get the IO*BlockStorageDriver node
-	err = IORegistryEntryGetChildEntry ( service, kIOServicePlane, &child );
-	( void ) IOObjectRelease ( service );
-	require ( ( err == kIOReturnSuccess ), ErrorExit );
+	bsdName = ( CFStringRef ) IORegistryEntrySearchCFProperty ( service,
+																kIOServicePlane,
+																CFSTR ( kIOBSDNameKey ),
+																kCFAllocatorDefault,
+																kIORegistryIterateRecursively );
+																
+	// If there isn't a property, no media exists for this device. Proceed as planned...
+	require_action ( ( bsdName != NULL ), ErrorExit, result = kStatusOK );
 	
-	// Check for existence of an IOMedia node below the IO*BlockStorageDriver node
-	err = IORegistryEntryGetChildEntry ( child, kIOServicePlane, &service );
-	( void ) IOObjectRelease ( child );
-	require ( ( err == kIOReturnSuccess ), ErrorExit );
-	require ( ( service != NULL ), ErrorExit );
-	
-	// There is an IOMedia object. We should find its name, and present a dialog
-	// to see if the user wants to unmount the volume.
-	result = [ self findWholeMediaBSDName: service ];
-	( void ) IOObjectRelease ( service );
-	
-	
-ErrorExit:
-	
-	
-	return result;
-	
-}
-
-
-//
-//	findWholeMediaBSDName
-//
-
-- ( int ) findWholeMediaBSDName : ( io_service_t ) ioMediaNub
-{
-	
-	CFMutableDictionaryRef	dict	= NULL;
-	CFStringRef				bsdName	= NULL;
-	IOReturn				err		= kIOReturnSuccess;
-	int						result	= kUnmountFailed;
-	
-	// Sanity Check
-	require ( ( IOObjectConformsTo ( ioMediaNub, kIOMediaClass ) ), ErrorExit );
-	
-	// Get the CF Properties for the IOMedia object
-	err = IORegistryEntryCreateCFProperties ( ioMediaNub, &dict, kCFAllocatorDefault, kNilOptions );
-	require ( ( err == kIOReturnSuccess ), ErrorExit );
-	
-	// Check for the BSD Name key
-	if ( CFDictionaryContainsKey ( dict, CFSTR ( kIOBSDNameKey ) ) )
-	{
-		
-		// Get the value for it
-		bsdName = ( CFStringRef ) CFDictionaryGetValue ( dict, CFSTR ( kIOBSDNameKey ) );
-		result = [ self unmountAllPartitions: bsdName ];
-		
-	}
-	
-	CFRelease ( dict );
+	// There was media. Unmount all the partitions associated with this device.
+	result = [ self unmountAllPartitions: bsdName ];
+	IOObjectRelease ( service );
+	CFRelease ( bsdName );
 	
 	
 ErrorExit:
@@ -381,111 +346,67 @@ ErrorExit:
 - ( int ) unmountAllPartitions: ( CFStringRef ) bsdName
 {
 	
-	OSStatus			error		= noErr;
-	struct statfs *		fsStats		= NULL;
-	int					result		= kUnmountFailed;
-	int					numMounts	= 0;
-	unsigned int		index		= 0;
-	FSRef				volFSRef	= { { 0 } };
-	FSCatalogInfo		volumeInfo	= { 0 };
+	DADiskRef			disk	= NULL;
+	DASessionRef		session	= NULL;
+	int					result	= kUnmountFailed;
 	
-	// Get the mount info for all the mountpoints. We will traverse that data
-	// and try and find any mountpoints which have the diskX identifier in them.
-	numMounts = getmntinfo ( &fsStats, MNT_NOWAIT );
-	require ( ( numMounts != 0 ), ErrorExit );
-	
-	for ( index = 0; index < numMounts; index++ )
+	session = DASessionCreate ( kCFAllocatorDefault );
+	if ( session != NULL )
 	{
 		
-		// make a CFStringRef with the name of the BSD dev node/file descriptor
-		CFStringRef cfStringMntOnName  = CFStringCreateWithCString (	NULL,
-																		&fsStats[index].f_mntfromname[5] /* Starting at [5] gets rid of the /dev/ */,
-																		CFStringGetSystemEncoding ( ) );
+		DASessionScheduleWithRunLoop ( session, CFRunLoopGetCurrent ( ), CFSTR ( "MyCallbackMode" ) );
 		
-		// Compare the BSD dev node to the one we have. This is akin to doing a strncmp.
-		if ( CFStringCompareWithOptions ( cfStringMntOnName,
-										  bsdName,
-										  CFRangeMake ( 0, CFStringGetLength ( bsdName ) ),
-										  0 ) == kCFCompareEqualTo )
+		disk = DADiskCreateFromBSDName ( kCFAllocatorDefault, session, CFStringGetCStringPtr ( bsdName, kCFStringEncodingMacRoman ) );
+		if ( disk != NULL )
 		{
 			
-			// Pop up a simple alert panel to warn the user that the disc will be unmounted
-			result = NSRunAlertPanel ( kAlertString,
-									   [ NSString stringWithFormat: @"Are you sure you want to unmount the disc: %s?", &fsStats[index].f_mntonname[9] ],
-									   kOKString,
-									   kCancelString,
-									   nil );
+			SInt32			returnValue = 0;
+			Context			context;
 			
-			if ( result == NSAlertAlternateReturn )
+			context.document 	= self;
+			context.result 		= kUnmountFailed;
+			
+			DADiskUnmount ( disk,
+							kDADiskUnmountOptionWhole,
+							DiskUnmountCallback,
+							&context );
+			
+			returnValue = CFRunLoopRunInMode ( CFSTR ( "MyCallbackMode" ), 20.0, false );
+			if ( ( returnValue == kCFRunLoopRunStopped ) && ( context.result == kStatusOK ) )
 			{
-				
-				// User cancelled unmount.
-				result = kUnmountCancelled;
-				CFRelease ( cfStringMntOnName );
-				cfStringMntOnName = NULL;
-				goto ErrorExit;
-				
-			}
+				result = kStatusOK;
+			}			
 			
-			// Ask Carbon to unmount the partition.
-			error = FSPathMakeRef ( fsStats[index].f_mntonname,
-									&volFSRef,
-									NULL );
-			if ( error != noErr )
-			{				
-				
-				result = kUnmountFailed;
-				CFRelease ( cfStringMntOnName );
-				cfStringMntOnName = NULL;
-				goto ErrorExit;
-				
-			}
-			
-			error = FSGetCatalogInfo (	&volFSRef,
-										kFSCatInfoVolume,
-										&volumeInfo,
-										NULL,
-										NULL,
-										NULL );
-			if ( error != noErr )
-			{
-				
-				result = kUnmountFailed;
-				CFRelease ( cfStringMntOnName );
-				cfStringMntOnName = NULL;
-				goto ErrorExit;
-				
-			}
-			
-			error = FSUnmountVolumeSync ( volumeInfo.volume, 0, NULL );
-			if ( error != noErr )
-			{
-				
-				result = kUnmountFailed;
-				CFRelease ( cfStringMntOnName );
-				cfStringMntOnName = NULL;
-				goto ErrorExit;
-				
-			}
+			CFRelease ( disk );
+			disk = NULL;
 			
 		}
 		
-		CFRelease ( cfStringMntOnName );
-		cfStringMntOnName = NULL;
+		DASessionUnscheduleFromRunLoop ( session, CFRunLoopGetCurrent ( ), CFSTR ( "MyCallbackMode" ) );
+		CFRelease ( session );
 		
 	}
 	
-	result = kStatusOK;
-	
-	
-ErrorExit:
-	
-	
 	return result;
 	
 }
 
 
+- ( void ) diskUnmountCallback: ( DADiskRef ) disk
+					 dissenter: ( DADissenterRef ) dissenter
+					   context: ( Context * ) context
+{
+	
+	if ( dissenter == NULL )
+	{
+		context->result = kStatusOK;
+	}
+	
+	CFRunLoopStop ( CFRunLoopGetCurrent ( ) );
+	
+}
+
+
 //
 //	runExclusiveTestSuite
 //
@@ -528,13 +449,13 @@ ErrorExit:
 	cdb[0] = kSCSICmd_READ_10;
 	
 	// Set the block to be the block zero
-	*( UInt32 * ) &cdb[2] = 0;
+	OSWriteBigInt32 ( &cdb[2], 0, 0 );
 	
 	// Tell the drive we only want this many blocks
-	*( UInt16 * ) &cdb[7] = kNumBlocks;
+	OSWriteBigInt16 ( &cdb[7], 0, kNumBlocks );
 	
 	// Set the actual cdb in the task
-	err = ( *task )->SetCommandDescriptorBlock ( task, cdb, kSCSICDBSize_6Byte );
+	err = ( *task )->SetCommandDescriptorBlock ( task, cdb, kSCSICDBSize_10Byte );
 	require ( ( err == kIOReturnSuccess ), ReleaseBuffer );
 	
 	// Set the scatter-gather entry in the task
@@ -630,7 +551,7 @@ ReleaseExclusiveAccess:
 	
 	
 	err = ( *interface )->ReleaseExclusiveAccess ( interface );
-	require ( ( err == kIOReturnSuccess ), ReleaseTask );
+	require ( ( err == kIOReturnSuccess ), ErrorExit );
 	
 	
 ErrorExit:
@@ -748,5 +669,19 @@ ErrorExit:
 	
 }
 
+@end
 
-@end
\ No newline at end of file
+
+static void
+DiskUnmountCallback ( DADiskRef disk, DADissenterRef dissenter, void * context )
+{
+	
+	Context *		myContext 	= nil;
+	MyDocument *	document	= nil;
+	
+	myContext 	= ( Context * ) context;
+	document	= myContext->document;
+	
+	[ document diskUnmountCallback: disk dissenter: dissenter context: myContext ];
+	
+}
\ No newline at end of file
